

/*********************************************************************
 *
 *		PCX.C										PCX.C
 *
 *	This file contains the functions used to read ".pcx" files, store
 * 	the data, and free it.
 *	The External interfaces are:
 *		Load_PCX()
 *		FreePic()
 *		WLoad_PCX()
 *		WFreePic()
 *
 **********************************************************************/
#include "generic.h"
#include "pcx.h"
#include "mymem.h"
#include <X11/Xlib.h>
#include <X11/Xutil.h>


/*********************************************************************
 *
 *	Load_PCX_data()									Load_PCX_data()
 *
 *	Loads data from a pcx file into the suplied buffer, the data
 *  in the file is Run Lenght Encoded (compressed), and represents
 * 	an image of width "xscan", and height "yscan".
 *
 */

static void 
Load_PCX_data (FILE * fp, uchar * where, int xscan, int yscan)
{
    int xcount;
    uchar buf, tbuf;
    while (yscan--) {
		xcount = 0;
		while (xscan > xcount) {
	    	buf = fgetc (fp);
	    	if (buf >= 192) {
				tbuf = buf - 192;
				xcount += tbuf;
				buf = fgetc (fp);
				while (tbuf--) {
		    		*(where++) = buf; 
				}
	    	} else {
				xcount++;
				*(where++) = buf;
	   		}
		}
    }
}

/**********************************************************************
 *
 *	SetupPT()											SetupPT()
 *
 *	allocates the memory required to store the picture data, and
 *  (if need be) palette. 
 *
 */

static struct Picture_Type *
SetupPT(int Buffsize, int Flags)
{
    uchar *where;
    struct Picture_Type *pic = NULL;
    where = (uchar *) MyMalloc (Buffsize, "Picture_Type(buffer)");
    if (where) {
		pic = (struct Picture_Type *) MyMalloc (sizeof (struct Picture_Type),
							"Picture_Type struct");
		if (pic) {
	    	pic->Data = where;
	    	if (Flags & LP_LOAD_PALETTE) {
				pic->Palette = (uchar *) MyMalloc (768, 
						   		"PictureType(Palette)");
				if (pic->Palette) {
		    		return (pic);
				}
	    	} else {
				pic->Palette = NULL;
				return (pic);
	    	}
	    	MyFree (pic, "Picture_Type struct");
		}
		MyFree (where, "Picture_Type(buffer)");
    }
    return (NULL);
}

/***********************************************************************
 *
 *	Load_PCX()												Load_PCX()
 *
 *	Loads a pcx file into a "picture" structure. The palette may 
 *	optionaly be loaded.
 *  This is used to load most of the graphics.
 *
 */

struct Picture_Type *
Load_PCX(char *name, int Flags)
{
    FILE *fp;
    int j;
    uchar *where;
    uchar *pal;
    int yscan, xscan, tmp;
    int topx, topy;
    struct Picture_Type *pic = NULL;
    fp = fopen (name, "rb");
    if (fp) {
		for (j = 0; j < 4; j++) {
	    	fgetc (fp);
		}
		topx = fgetc (fp) + (fgetc (fp) << 8);
		topy = fgetc (fp) + (fgetc (fp) << 8);
		xscan = fgetc (fp) + (fgetc (fp) << 8) + 1;
		yscan = fgetc (fp) + (fgetc (fp) << 8) + 1;
		yscan -= topy;
		xscan -= topx;
		pic = SetupPT (xscan * yscan, Flags);
		if (pic) {
	    	where = pic->Data;
	    	pic->Width = xscan;
	    	pic->Height = yscan;
	    	tmp = xscan >> 1;
	    
	    	for (pic->shift = 0; tmp; tmp >>= 1, pic->shift++)
				;
	    	fseek (fp, 128, SEEK_SET);
	    	Load_PCX_data (fp, where, xscan, yscan);
	    	if (Flags & LP_LOAD_PALETTE) {
				pal = pic->Palette;
				fseek (fp, -768, SEEK_END);
				for (j = 0; j < 768; j++) {
		    		*(pal++) = fgetc (fp);
				}
	    	}
		} else {
	    	fprintf (stderr, "ERROR: Load_PCX() - memory\n");
		}
		fclose (fp);
   	} else {
		fprintf (stderr, "ERROR: Load_PCX() - file (%s)\n", name);
	}
    return (pic);
}

/*********************************************************************
 *
 * 	FreePic()											FreePic()
 *
 *	Frees the memory associated with the picture, previously allocated
 *  by Load_PCX()
 *
 */ 

void FreePic(struct Picture_Type *pic)
{
    if(pic) {
		if(pic->Palette) {
	    	MyFree (pic->Palette, "Picture_Type(Palette)");
		}
		if(pic->Data) {
	    	MyFree (pic->Data, "Picture_Type(Data)");
		}
		MyFree (pic, "Picture_Type(struct)");
    }
}

/************************************************************************
 *
 *	WLoad_PCX()												WLoad_PCX()
 *
 *	Load a pcx file.   Loads the file into an XImage structure.
 *
 */


XImage *WLoad_PCX (char *fname, int flags, Display * dis, int paletteoffset)
{
    FILE *fp;
    XImage *image = NULL;
    char *mydata;
    int topx, topy, xscan, yscan, j;
    fp = fopen (fname, "r");
    if (fp) {
		for (j = 0; j < 4; j++)
			fgetc (fp);
		topx = fgetc (fp) + (fgetc (fp) << 8);
		topy = fgetc (fp) + (fgetc (fp) << 8);
		xscan = fgetc (fp) + (fgetc (fp) << 8) + 1;
		yscan = fgetc (fp) + (fgetc (fp) << 8) + 1;
		yscan -= topy;
		xscan -= topx; 
		//fprintf(stdout,"%s: %d %d\n",fname,xscan,yscan);
		mydata = MyMalloc (xscan * yscan, "ximage-data (WLoad_PCX())");
		if (mydata) { 
			image = XCreateImage (dis, 
					  XDefaultVisual (dis, DefaultScreen (dis)),
					  8, ZPixmap, 0, mydata, xscan, yscan, 32, 
					  xscan);  
			if (image) {
				fseek (fp, 128, SEEK_SET);
				Load_PCX_data (fp, mydata, xscan, yscan);
				XAddPixel (image, paletteoffset);
				fclose (fp);  
				return (image); 
			}
			MyFree (mydata, "ximage-data (WLoad_PCX())");
		}
		fclose (fp);  
	} else {
		fprintf(stdout,"Can't find file %s\n",fname);
	}
	return (NULL);
}

/**** BOLLIX ***********************************************************
 *
 * 	LoadStatsPic()										LoadStatsPic()
 *
 *	This function doesn't live here.....
 *
 */

XImage *LoadStatsPicture(Display *dis,int basepix)
{
	XImage *stImage;	
	if(!(stImage=WLoad_PCX("textures/logo.pcx",0,dis,basepix))) {
		fprintf(stdout,"Ouch pcx.c:%d\n",__LINE__);
	}
	return(stImage);
}

/***********************************************************************
 *
 *	WFreePic()										WFreePic()
 *
 *	Frees memory assosiated with an XImage, previously allocated by
 *	WLoad_PCX()
 *
 */

void WFreePic (XImage * pic)
{
    if (pic) {
		XDestroyImage (pic);
    }
}

