

/***********************************************************************
 *
 *	SOUND.C													SOUND.C
 *
 *	All the routines for loading/playing/unloading sampled sounds.
 *
 *	The public interfaces are:
 *		SoundManager()
 *		PlaySound()
 *		WaitForSilence()
 *		QueryVlume()
 *		SetVolume()
 *		CloseSoundManager()
 *
 ***********************************************************************/


#include "sound.h"
#include <sys/audioio.h>
#include <stdio.h>
#include <unistd.h>
#include <strings.h>
#include "mymem.h"

/* Static protos don't want to go in the .h file */
static void FreeSound(struct Sound *sound);
static void FreeSounds(struct Sound *sounds[],int num);
static struct Sound **LoadSounds (char *sounds[], int num);
static struct Sound *LoadSound(char *name);
static FILE *audio;
static FILE *audioctl;
static int audioctl_des;

#define uchar unsigned char
#define ROUND_VAL 2*8192	// To pad samples to the buffer size.
#define RoundUp(x,y) ((((x)+(y)-1)/(y))*(y))

/********************************************************************
 *
 *	PlaySound()											PlaySound()
 *
 *	Attempts to play a sampled sound.
 *	The sound will only be played if the audio device tells us that
 *	there is no previous sound playing still.
 *
 */

void PlaySound(struct Sound *snd)
{
	audio_info_t audioinfo;
	if(!snd) {
		fprintf(stdout,"Attempt to play null sound\n");
		return;
	}
	if(!ioctl(audioctl_des,AUDIO_GETINFO,&audioinfo)) {
		if(!audioinfo.play.active) {
			fwrite(snd->Data,snd->Length,1,audio);
			return;
		}
	}
}

/********************************************************************
 *
 *	WaitForSilence()								WaitForSilence()
 *
 *	Waits for the currently playing sample (if there is one) to stop.
 *	returns 1 when the sound has stopped, or 0 if the audio device
 *	can't tell us.
 *
 */

int WaitForSilence(void)
{
	audio_info_t audioinfo;
	int done=0,err;
	while(!done) {
		err=ioctl(audioctl_des,AUDIO_GETINFO,&audioinfo);
		if(err) {	
			done=1; // Can't tell.
		} else {
			if(!audioinfo.play.active) {
				done=2;  // Got silence.
			}
		}
	}
	return(done-1);
}

/**************************************************************************
 *
 *	QueryVolume() 											QueryVolume()
 *
 *	Asks the audio device what volume sounds are set to be played back at.
 *	returns the volume as a percentage of the loudest possible.
 *
 */

int 
QueryVolume(void)
{
	int err,v;
	audio_info_t audioinfo;
	err=ioctl(audioctl_des,AUDIO_GETINFO,&audioinfo);
	if(!err) {
		v=((audioinfo.play.gain)*100)/255;
		return(v);
	}
	return(0);
}

/************************************************************************
 *
 *	SetVolume()											SetVolume()
 *
 *	Sets the volume at which sounds are to be played back at. the
 *	parameter is assumed to be a percentage, and is converted 
 *	accordingly.
 *
 */

void 
SetVolume(int newvol)
{
 	audio_info_t info;
	int err;
	newvol=(newvol*255)/100;
	AUDIO_INITINFO(&info);
	info.play.gain=newvol;
  	err = ioctl(audioctl_des, AUDIO_SETINFO, &info);
	if(err) {
		fprintf(stdout,"Can't set volume\n");
	}
}

/***********************************************************************
 *
 *	SoundManager()										SoundManager()
 *
 *	Sets up the sound playback functions. 
 *	Opens the "/dev/audio" and "/dev/audioctl" devices,
 *	and loads the smaples named be the parameter names.
 *
 */

struct Sound **
SoundManager(char *names[],int num)
{
	struct Sound **sounds;
    audio = fopen("/dev/audio", "wb");
    if(audio) {
		audioctl=fopen("/dev/audioctl","r");
		if(audioctl) {
			audioctl_des=fileno(audioctl);
			sounds=LoadSounds(names,num);
			if(sounds) {
				return(sounds);
			}
			fclose(audioctl);
		}
		fclose(audio);
    }
	fprintf(stdout,"Sound manager not installed\n");
	return(NULL);
}

/*********************************************************************
 *
 *	LoadSound()											LoadSound()
 *
 *	Loads a ".au" file, pads it to the soundcards buffer size.
 *
 */

static struct Sound *
LoadSound(char *name)
{
	FILE *fp;
	int len, filelen,i,start;
	struct Sound *sound=NULL;
	fp=fopen(name,"rb");
	if(fp) {
		fseek(fp,0,SEEK_END);
		filelen=ftell(fp);
		fseek(fp,7,SEEK_SET);
		start=fgetc(fp);
		fseek(fp,start,SEEK_SET);
		filelen-=start;
        len=RoundUp(filelen,ROUND_VAL)+1;
		sound=MyMalloc(sizeof(struct Sound),"struct Sound");
		if(sound) {
			sound->Length=0;
			sound->Data=MyMalloc(len,"Sound buff");
			if(sound->Data) {
				for(i=0;i<len;i++) {
					sound->Data[i]=0x7F;
				}
				fread(sound->Data,filelen,1,fp);
				sound->Length=len;
			}
		}
		fclose(fp);
		return(sound);
	} else {
		fprintf(stdout,"File not found: %s\n",name);
	}
	return(NULL);
}

/************************************************************************
 *
 *	LoadSounds()										LoadSounds()
 *
 *	Loads all sounds required by the game.
 *
 */

static struct Sound **
LoadSounds(char *sounds[], int num)
{
    struct Sound **array;
    int i;
    array = MyMalloc (num * sizeof (struct Sound *), "sound array");
    if (array) {
		for (i = 0; i < num; i++) {
	    	array[i]=LoadSound(sounds[i]);
		}
    }
    return (array);
}

/**********************************************************************
 *
 *	FreeSound()										FreeSound()
 *
 * 	Frees memory previously allocated be a call to LoadSound()
 *
 */

static void 
FreeSound(struct Sound *sound)
{
	if(sound) {
		if(sound->Data) {
			MyFree(sound->Data,"sound buff");
		}
		MyFree(sound,"sound struct");
	}
}

/**********************************************************************
 *
 *	FreeSounds()									FreeSounds()
 *
 *	Frees the memory used to store all of the loaded sounds.
 *
 */

static void 
FreeSounds(struct Sound *sounds[],int num)
{
	int i;
	for(i=0;i<num;i++) {
		FreeSound(sounds[i]);
	}
}

/*********************************************************************
 *
 *	CloseSoundManager()						CloseSoundManager()
 *
 *	Returns all resources allocated by this module (devices/memory)
 *	to the system.
 *
 */

void
CloseSoundManager(struct Sound *sounds[],int num)
{
	FreeSounds(sounds,num);
	if(audioctl) {
		fclose(audioctl);
		if(audio) {
			fclose(audio);
		}
	}
}

